﻿#version 440

struct Material
{
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
	sampler2D diffuseTex;
	sampler2D specularTex;
};



in vec3 vsPos;
in vec3 vsColor;
in vec2 vsTexCoord;
in vec3 vsNormal;
in float vsSteepness;
in float vsHeightAdjust;
in vec3 vsModelPos;

out vec4 fsColor;

uniform Material material;

uniform vec3 lightPos0;
uniform vec3 cameraPos;
uniform vec3 planetPos;




// Colouring
uniform float planetRadius;
uniform float maxFlatHeight;
uniform float shoreHeight;
uniform float shoreBlend;
uniform float steepThreshold;
uniform float steepBlend;
uniform float oceanBlend;

uniform vec3 grassCol;
uniform vec3 shoreCol;
uniform vec3 steepCol;
uniform vec3 deepOceanCol;
uniform vec3 shallowOceanCol;


float near = 0.1; 
float far  = 100.0;

float lerp(float a, float b, float t)
{
	return a + ((b - 2) * t);
}

vec3 lerp(vec3 a, vec3 b, float t)
{
	return a * (1 - t) + b * t;
}


vec3 CalculateLighting()
{
	// Ambient Light
	vec3 ambientLight = material.ambient;


	// Diffuse Light
	vec3 posToLightNorm = normalize(lightPos0 - vsPos);
	vec3 diffuseColor = material.diffuse;
	float diffuse = clamp(dot(posToLightNorm, vsNormal), 0, 1);

	vec3 diffuseLight = diffuseColor * diffuse;

	// Specular Light
	vec3 lightToPosNorm = normalize(vsPos - lightPos0);
	vec3 reflectDir = normalize(reflect(lightToPosNorm, normalize(vsNormal)));
	vec3 posToCamNorm = normalize(cameraPos - vsPos);
	float specularStrength = 10;
	float specularConstant = pow(max(dot(posToCamNorm, reflectDir), 0), specularStrength);

	vec3 specularLight = material.specular * specularConstant;

	return (ambientLight + diffuseLight + specularLight);
}




#if 0
vec3 GetFragmentColour()
{
	// Calculate heights
	float terrainHeight = length(vsModelPos);

	// Water + Coast
	if (terrainHeight < 1.00001)
	{
		// 1.0 is highest ocean can be
		// 0.8 is lowest ocean can be
		// 0.2 room of blending
		vec3 returnColour = lerp(deepOceanCol, shallowOceanCol, vsHeightAdjust);

		// Coast Blending
		if (terrainHeight > 1 - oceanBlend)
		{
			float transition = (terrainHeight - (1 - oceanBlend)) / (oceanBlend * 3); // startingHeight = shoreHeight // transitionWidth = shoreBlend;
			returnColour = lerp(returnColour, shoreCol, transition);
		}

		return returnColour;
	}
	// Cliffs / Mountains
	if (vsSteepness > (steepThreshold + steepBlend))
	{
		return steepCol;
	}
	// Coast
	if (terrainHeight < shoreHeight)
	{
		return shoreCol;
	}
	// Grass + Coast
	if (terrainHeight < shoreHeight + shoreBlend)
	{
		float transition = (terrainHeight - shoreHeight) / shoreBlend; // startingHeight = shoreHeight // transitionWidth = shoreBlend;
		return lerp(shoreCol, grassCol, transition);
	}
	// Grass
	if (vsSteepness < steepThreshold && terrainHeight < 1 + maxFlatHeight)
	{
		return grassCol;
	}

	// Grass + Mountain
	float maxGrassSteepHeight = 1 + maxFlatHeight;// +steepBlend;
	
	if (vsSteepness < (steepThreshold + steepBlend) && terrainHeight < maxGrassSteepHeight)
	{
		float transition = (vsSteepness - steepThreshold) / (steepBlend * 2); // startingHeight = shoreHeight // transitionWidth = shoreBlend;
		return lerp(grassCol, steepCol, transition);
	}

	// Mountain
	return steepCol;
}
#endif // 0



void main()
{
	// Colour
	fsColor = vec4(vsColor, 1.0f);
	//fsColor = vec4(GetFragmentColour(), 1);

	// Texture
	vec3 lighting = CalculateLighting();

	// Lighting
	fsColor *= vec4(lighting, 1.0f);

}
