#version 440

layout (location = 0) in vec3 vertexPos;
layout (location = 1) in vec3 vertexColor;
layout (location = 2) in vec2 vertexTexCoord;
layout (location = 3) in vec3 vertexNormal;

out vec3 vsPos;
out vec3 vsColor;
out vec2 vsTexCoord;
out vec3 vsNormal;

out vec3 vsModelPos;

out float vsSteepness;
out float vsHeightAdjust;

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix; // Camera



// Colouring
uniform float planetRadius; 
uniform float maxFlatHeight; 
uniform float shoreHeight;
uniform float shoreBlend;
uniform float steepThreshold;
uniform float steepBlend;
uniform float oceanBlend;

uniform vec3 grassCol;
uniform vec3 shoreCol;
uniform vec3 steepCol;
uniform vec3 deepOceanCol;
uniform vec3 shallowOceanCol;


uniform int erosionTest;

// These three functions are taken from online sources
// Original source could not be found,
// however these functions are often used in shaders on shadertoy.com
#define sat(x) clamp(x,0.,1.) 
float remap01(float a, float b, float t)
{
	return sat((t - a) / (b - a));
}

float lerp(float a, float b, float t)
{
	return a + ((b - 2) * t);
}

vec3 lerp(vec3 a, vec3 b, float t)
{
	return a * (1 - t) + b * t;
	return a + ((b - 2) * t);
}


#if 1
vec3 GetVertexColour()
{
	vec3 sphereNormal = normalize(vertexPos);
	vsSteepness = 1 - dot(vertexNormal, sphereNormal);
	vsSteepness = sat(vsSteepness / 0.6);

#if 1
	// Calculate heights
	float terrainHeight = length(vertexPos);
	float aboveShoreHeight = remap01(terrainHeight, 1, 1.1);
	float flatHeight = sat(aboveShoreHeight / maxFlatHeight); //commenting this out removes shores

#if 1
	if (terrainHeight > 1.13)
	{
		return vec3(1.0f);
	}

	if (vsSteepness > 0.4 && terrainHeight > 1.0)
	{
		return steepCol;
	}

	if (terrainHeight < 1.00001)
	{
		// 1.0 is highest ocean can be
		// 0.8 is lowest ocean can be
		// 0.2 room of blending
		float oceanDepth = (terrainHeight - 0.7) / 0.3;
		oceanDepth *= oceanDepth;
		return lerp(deepOceanCol, shallowOceanCol, oceanDepth);
	}
	if (terrainHeight < 1 + oceanBlend)
	{
		float transition = (terrainHeight - 1) / oceanBlend; // startingHeight = shoreHeight // transitionWidth = shoreBlend;
		return lerp(shallowOceanCol, shoreCol, transition);
	}
	if (terrainHeight < shoreHeight)
	{
		return shoreCol;
	}
	if (terrainHeight < shoreHeight + shoreBlend)
	{
		float transition = (terrainHeight - shoreHeight) / shoreBlend; // startingHeight = shoreHeight // transitionWidth = shoreBlend;
		return lerp(shoreCol, grassCol, transition);
	}
	else if (vsSteepness < 0.2 && terrainHeight < 1 + maxFlatHeight)
	{
		return grassCol;
	}
	else
	{
		return steepCol;
	}
#endif

	// Biome
	vec3 flatTerrainCol = grassCol;

	// Shore
	float shoreBlendWeight = 1 - mix(shoreHeight, shoreBlend, flatHeight); // This is broken
	flatTerrainCol = lerp(flatTerrainCol, shoreCol, shoreBlendWeight);

	// Steep terrain colour
	vec3 steepTerrainCol = steepCol;

	// Set surface colour
	float flatStrength = 0.5;

	vec3 compositeCol = lerp(steepTerrainCol, flatTerrainCol, flatStrength);
	vsColor = compositeCol;

	if (vsSteepness > 0.3)
	{
		compositeCol = steepCol;
	}

	return compositeCol;
#endif
}
#endif // 0



void main()
{
	vec3 sphereNormal = normalize(vertexPos);
	vsSteepness = 1 - dot(vertexNormal, sphereNormal);
	vsSteepness = sat(vsSteepness / 0.6);

	vsColor = GetVertexColour();

	if (vertexColor != vec3(1))
	{
		//vsColor = vertexColor;
	}
	//vsColor = vertexColor;

	vsModelPos = vertexPos;
	vec3 alteredVertexPos = vertexPos;
	vec3 alteredVertexNormal = vertexNormal;

	// Push up any vertices with a height lower than 1
	// These will be out oceans
	if (length(alteredVertexPos) < 1 && (erosionTest == 0))
	{
		alteredVertexPos = normalize(alteredVertexPos);
		alteredVertexNormal = normalize(alteredVertexPos);

		float oceanDepth = (length(vsModelPos) - 0.7) / 0.3;
		oceanDepth *= oceanDepth;

		vsHeightAdjust = oceanDepth;
	}

	alteredVertexPos *= planetRadius;

	vsPos = vec4(modelMatrix * vec4(alteredVertexPos, 1.0f)).xyz;

	vsTexCoord = vec2(vertexTexCoord.x, vertexTexCoord.y * -1.0f);

	vsNormal = mat3(modelMatrix) * alteredVertexNormal;

	gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(alteredVertexPos, 1.0f);
}